import MapBase from "./MapBase";
import * as Utils from "./utils";

import { connect } from "react-redux";

class LotMap extends MapBase {
  componentDidUpdate(prevProps) {
    const { lot } = this.props;
    if (lot !== prevProps.lot) {
      this.center(lot.position);
      this.map.setZoom(16);
    }
  }

  draw() {
    const { lot, authUser } = this.props;

    if (!lot || !lot.position) return;

    const { position = {} } = lot;
    const lng = position._longitude;
    const lat = position._latitude;
    const zoom = this.map.getZoom().toFixed(2);

    this.setState({ lng, lat, zoom, loading: true });

    this.center({ lng, lat });

    this.bubble({
      position,
      inner: Object.keys(lot.cells).length,
      type: Utils.getLotType(lot, authUser)
    }).addTo(this.map);

    this.setState({ loading: false });
  }

  onMapLoad() {
    super.onMapLoad();
    this.draw();
  }
}

const mapStateToProps = state => {
  const { authUser } = state.session;
  return { authUser };
};

export default connect(mapStateToProps)(LotMap);
