import React from "react";
import { withFirebase } from "../Firebase";
import { STATUS } from "../../constants/lot";
import * as FORMAT from "../../utils/Formater";

const gtag = window.gtag;

class LotPopup extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      lot: this.preparePopupData(props.lots)
    };
  }

  componentWillReceiveProps(data) {
    this.setState({
      lot: this.preparePopupData(data.lots)
    });
  }

  preparePopupData(lots) {
    if (!lots) return false;

    const keys = Object.keys(lots);
    const id = keys[0];
    const count = keys.length;
    const lot = lots[id];
    const addr = FORMAT.addressByLocale((lot.route || {}).address);
    const position = FORMAT.geoCoords(lot.position);
    const price = FORMAT.money(keys.map(v => lots[v].price).sum());
    const status = lots[id].status;
    const rentPrice = FORMAT.money(keys.map(v => lots[v].rentPrice).sum());

    return { id, addr, price, count, position, status, rentPrice };
  }

  buy() {
    const { lot } = this.state;
    const { firebase, canBuy, callback } = this.props;
    const price =
      parseFloat((lot.price.toString() || "0").replace(/[^\d\.]+/g, "")) * 1;
    gtag("event", "buyClick", {
      event_category: "ecommerce",
      event_label: "popupButtonBuy",
      value: price
    });

    if (!canBuy) return;

    this.setState({
      lot: false
    });

    firebase
      .buyLot({
        [lot.id]: {}
      })
      .then(lot => {
        gtag("event", "buy", {
          event_category: "ecommerce",
          event_label: "popupButtonBuy",
          value: price
        });
        callback();
      })
      .catch(error => {
        console.error(error);
      });
  }
  rent() {
    const { lot } = this.state;
    const { firebase, canBuy, callback } = this.props;
    const price =
      parseFloat((lot.rentPrice.toString() || "0").replace(/[^\d\.]+/g, "")) *
      1;
    gtag("event", "rentClick", {
      event_category: "ecommerce",
      event_label: "popupButtonBuy",
      value: price
    });

    if (!canBuy) return;

    this.setState({
      lot: false
    });
    firebase
      .rentLot({
        lotid: lot.id,
        period: 1
      })
      .then(lot => {
        gtag("event", "rent", {
          event_category: "ecommerce",
          event_label: "popupButtonBuy",
          value: price
        });
        callback();
      });
  }

  render() {
    const { lot } = this.state;

    return (
      <div id="lot-popup-content">
        {!lot && <div>Loading ...</div>}
        {lot && (
          <div>
            <p className="popup-addres1">{lot.addr.address}</p>
            <p className="popup-addres2">{lot.addr.region}</p>
            <p className="popup-lotposition">{lot.position}</p>
            <hr />
            <span className="popup-count">{`${FORMAT.plural(
              lot.count,
              "lot"
            )} selected`}</span>
            {lot.status == STATUS.FOR_SALE && (
              <div className="btn btn-blue" onClick={this.buy.bind(this)}>
                {lot.price} Buy
              </div>
            )}
            {lot.status == STATUS.FOR_RENT && (
              <div className="btn btn-black" onClick={this.rent.bind(this)}>
                {lot.rentPrice} Rent
              </div>
            )}
          </div>
        )}
      </div>
    );
  }
}

export default withFirebase(LotPopup);
