import React, { Fragment } from "react";
import { connect } from "react-redux";
import mapboxgl from "mapbox-gl";

import * as Utils from "./utils";

mapboxgl.accessToken =
  "pk.eyJ1IjoibWFwYm94IiwiYSI6ImNpejY4M29iazA2Z2gycXA4N2pmbDZmangifQ.-g_vE53SD2WrJ6tFX7QHmA";

export const DEFAULT_STATE = {
  lng: 37.635308,
  lat: 55.737694,
  zoom: 16,
  loading: false
};
export const ERRORS = {
  NO_POSITION: "Lot position is not defined"
};

const MAX_ZOOM = 18;
const MIN_ZOOM = 3;

class MapBase extends React.Component {
  constructor(props) {
    super(props);

    this.state = { ...DEFAULT_STATE, error: null };
  }

  center(position = {}) {
    if (this.map) {
      if (position instanceof Array) {
        const [lng = this.state.lng, lat = this.state.lat] = position;
        this.map.setCenter([lng, lat]);
      } else {
        const { lng = this.state.lng, lat = this.state.lat } = position;
        this.map.setCenter([lng, lat]);
      }
    }
  }

  bubble({
    type,
    position = {},
    feature = {},
    width = 50,
    height = 50,
    inner,
    listeners = {}
  }) {
    const el = document.createElement("div");
    el.className = "marker";
    el.style.backgroundImage = "url(" + Utils.bubbles_colors[type].bubble + ")";
    el.style.width = width + "px";
    el.style.height = height + "px";

    el.feature = feature;

    for (let event in listeners) {
      el.addEventListener(event, listeners[event].bind(this));
    }

    if (inner) {
      const innerSpan = document.createElement("span");
      innerSpan.className = "marker";
      innerSpan.innerText = inner;

      el.appendChild(innerSpan);
    }

    let lng, lat;
    if (position instanceof Array) {
      [lng, lat] = position;
    } else {
      lng = position._longitude || position.lng || position.lon;
      lat = position._latitude || position.lat;
    }

    const marker = new mapboxgl.Marker(el);

    if (lng && lat) marker.setLngLat([lng, lat]);

    return marker;
  }

  onMapLoad() {
    console.log("MAP WAS LOADED");
    console.log(
      JSON.stringify(
        {
          "this.map._canvas.clientHeight": this.map._canvas.clientHeight,
          "this.map._container.clientHeight": this.map._container.clientHeight,
          "this.map._canvasContainer.clientHeight": this.map._canvasContainer
            .clientHeight
        },
        null,
        2
      )
    );
  }
  onMapMoveEnd(data) {
    console.log("MAP MOVE ENDED", data);
    this.controlHeight();
    console.log(
      JSON.stringify(
        {
          "this.map._canvas.clientHeight": this.map._canvas.clientHeight,
          "this.map._container.clientHeight": this.map._container.clientHeight,
          "this.map._canvasContainer.clientHeight": this.map._canvasContainer
            .clientHeight
        },
        null,
        2
      )
    );
  }

  controlHeight() {
    const height = this.map._canvas.clientHeight + 2;
    const width = this.map._canvas.clientWidth + 2;
    this.map._container.style.height = `${height}px`;
    this.map._canvasContainer.style.height = `${height}px`;
    this.map._container.style.width = `${width}px`;
    this.map._canvasContainer.style.width = `${width}px`;
  }

  componentDidMount() {
    const { lng, lat, zoom } = this.state;

    this.map = new mapboxgl.Map({
      container: this.mapContainer,
      style: "mapbox://styles/mapbox/streets-v11",
      maxZoom: MAX_ZOOM,
      minZoom: MIN_ZOOM,
      center: [lng, lat],
      zoom
    });

    this.map.on("load", this.onMapLoad.bind(this));

    this.map.on("moveend", this.onMapMoveEnd.bind(this));
    this.map.on("resize", (...args) => {
      console.log("MAP RESIZE", args);
      console.log(
        JSON.stringify(
          {
            "this.map._canvas.clientHeight": this.map._canvas.clientHeight,
            "this.map._container.clientHeight": this.map._container
              .clientHeight,
            "this.map._canvasContainer.clientHeight": this.map._canvasContainer
              .clientHeight
          },
          null,
          2
        )
      );
    });
  }

  render() {
    const { lng, lat, zoom, loading } = this.state;
    const { style = {} } = this.props;
    const informer = false;
    return (
      <Fragment>
        <div
          className={`flex-1 map-container${loading ? " loading" : ""}`}
          style={{ ...style }}
          ref={el => (this.mapContainer = el)}
        >
          {informer && (
            <div
              className="informer"
              style={{ zIndex: 999 }}
            >{`Longitude: ${lng} Latitude: ${lat} Zoom: ${zoom}`}</div>
          )}
        </div>
      </Fragment>
    );
  }
}

export default MapBase;
