import React from "react";
import { connect } from "react-redux";
import { compose } from "recompose";

import { Segment, Message } from "semantic-ui-react";
import uuidv1 from "uuid/v1";

import LotCard from "./LotCard";

import { withFirebase } from "../Firebase";

import { REFRESH } from "../../constants/lot";

import * as BIT from "../../utils/Bitwise";

import "./Lots.scss";

class UserLotsList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      top5: true,
      lots: [],
      loading: false,
      error: null
    };
  }

  switchTop5() {
    const { top5 } = this.state;
    this.setState({ top5: !top5 });
  }

  header(top5) {
    return (
      <div className="lots-list_header flex">
        <div className="lots-list_title grey">{`${
          top5 ? "Top 5 user lots" : "User lots"
        }`}</div>
        <div
          className="lots-list_switcher blue"
          onClick={this.switchTop5.bind(this)}
        >{`${top5 ? "View all" : "View top 5"}`}</div>
      </div>
    );
  }

  componentDidMount() {
    this.fetch();
  }

  componentDidUpdate() {
    if (BIT.isset(this.props.needRefresh, REFRESH.LIST)) {
      console.log("NEED REFRESH LIST");
      this.fetch();
    }
  }

  fetch() {
    const { firebase, authUser } = this.props;

    this.setState({ loading: true, error: null });

    const ref = firebase.db.ref("lots");
    Promise.all([
      ref
        .orderByChild("owner")
        .equalTo(authUser.uid)
        .once("value"),
      ref
        .orderByChild("renter")
        .equalTo(authUser.uid)
        .once("value")
    ])
      .then(snapshots => {
        let val = { ...snapshots[0].val(), ...snapshots[1].val() };
        const lots = Object.keys(val).map(uid => ({ ...val[uid], uid }));
        this.setState({ lots });
      })
      .catch(error => {
        this.setState({ error });
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  }

  render() {
    const { lots, loading, error, top5 } = this.state;

    const len = lots.length;
    const filtered = top5 === true ? lots.slice(0, 5) : lots;
    return (
      <Segment className="lots-list" loading={loading}>
        {(show => (show ? this.header(top5) : null))(len > 5)}
        {error && <Message negative content={error}></Message>}
        {filtered.map(lot => (
          <LotCard lot={lot} key={uuidv1()} />
        ))}
      </Segment>
    );
  }
}

const mapStateToProps = state => {
  const { needRefresh } = state.lot;
  const { authUser } = state.session;

  return {
    needRefresh,
    authUser
  };
};

export default compose(
  withFirebase,
  connect(mapStateToProps)
)(UserLotsList);
