import React from "react";
import { connect } from "react-redux";
import { compose } from "recompose";
import { Redirect, withRouter } from "react-router-dom";

import * as ROUTES from "../../constants/routes";

import UserLotsList from "../../components/UserLotsList";
import UserDataBlock from "../../components/UserDataBlock";
import Map from "../../components/Mapbox/Map";
import StatusFilters from "../../components/StatusFilters";

import { setActiveRoute } from "../../actions/page";

class HomePage extends React.Component {
  componentDidMount() {
    const { setActiveRoute } = this.props;
    setActiveRoute(ROUTES.HOME);
  }

  render() {
    const { authUser } = this.props;

    if (!authUser) {
      return <Redirect to={ROUTES.SIGN_IN} />;
    }
    return (
      <div className="page table">
        <div className="table-row">
          <div className="table-cell" style={{ width: "65%" }}>
            <StatusFilters />
          </div>
          <div className="table-cell" style={{ width: "35%" }}>
            <UserDataBlock />
          </div>
        </div>
        <div className="table-row">
          <div className="table-cell" style={{ width: "65%" }}>
            <Map />
          </div>
          <div className="table-cell" style={{ width: "35%" }}>
            <UserLotsList />
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = state => {
  const { authUser } = state.session;
  return { authUser };
};

const mapDispatchToProps = dispatch => ({
  setActiveRoute: route => dispatch(setActiveRoute(route))
});

export default compose(
  withRouter,
  connect(
    mapStateToProps,
    mapDispatchToProps
  )
)(HomePage);
