import React, { Fragment } from "react";
import { connect } from "react-redux";
import { compose } from "recompose";
import { withFirebase } from "../../../components/Firebase";
import uuidv1 from "uuid/v1";

import { Button, Label, Form, Radio } from "semantic-ui-react";

import * as ACTIONS from "../../../actions/lot";

import * as FORMAT from "../../../utils/Formater";

const PERIODS = {
  Day: 1,
  Week: 7,
  Month: 30
};

class RentButton extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      days: 0,
      min: 0,
      max: 0,
      Balance: 0,
      rentPrice: 0,
      total: 0,
      valid: false
    };
  }

  componentDidMount() {
    const maxValidIndex = (arr, value) => {
      let idx = arr.findIndex(v => v > value) - 1;
      return Math.max(0, idx === -2 ? arr.length - 1 : idx);
    };
    const { Balance } = this.props.authUser;
    const { rentPrice } = this.props.lot;
    const periods = Object.values(PERIODS);
    const days =
      periods[maxValidIndex(periods.map(v => v * rentPrice), Balance)] || 0;
    const total = days * rentPrice;
    const valid = total <= Balance;
    this.setState({ days, total, valid });
  }

  onclick(e) {
    const {
      firebase,
      lot,
      setLotStatusStart,
      setLotStatusSuccess,
      setLotStatusError
    } = this.props;
    const { days, valid } = this.state;

    if (e.target.tagName === "INPUT" || valid === false) return false;
    e.preventDefault();

    setLotStatusStart();
    firebase
      .rentLot({ lotid: lot.uid, period: days })
      .then(response => {
        try {
          const data = JSON.parse(response.data.lots)[lot.uid];
          setLotStatusSuccess({ ...data, uid: lot.uid });
        } catch (error) {
          setLotStatusError(error);
        }
      })
      .catch(error => {
        setLotStatusError(error);
      });
  }

  onchange(e, { value }) {
    const { rentPrice } = this.props.lot;
    const { Balance } = this.props.authUser;
    const days = parseInt(value, 10);
    const total = days * rentPrice;
    const valid = total <= Balance;

    this.setState({ days, total, valid });
  }

  stopEvent(e) {
    e.stopPropagation();
  }

  render() {
    const { color } = this.props;
    const { days, total, valid } = this.state;

    return (
      <Fragment>
        <Button
          as="div"
          labelPosition="right"
          onClick={this.onclick.bind(this)}
        >
          <Button color={valid === true ? color : "grey"}>Rent</Button>
          <Label
            as="a"
            basic
            color={color}
            pointing="left"
            onClick={this.stopEvent}
            style={{ cursor: "default" }}
          >
            <Form>
              <Form.Group inline style={{ margin: 0 }}>
                {Object.keys(PERIODS).map(key => (
                  <Form.Field
                    key={uuidv1()}
                    control={Radio}
                    label={key}
                    value={PERIODS[key]}
                    checked={days === PERIODS[key]}
                    onChange={this.onchange.bind(this)}
                  />
                ))}
              </Form.Group>
            </Form>
            <div
              style={{
                whiteSpace: "nowrap",
                color: valid ? "green" : "red"
              }}
            >
              {FORMAT.money(total)}
            </div>
          </Label>
        </Button>
        <div />
      </Fragment>
    );
  }
}

const mapStateToProps = state => {
  const { authUser } = state.session;
  const { currentLot } = state.lot;

  return { authUser, lot: currentLot };
};
const mapDispatchToProps = dispatch => ({
  setLotStatusStart: () => dispatch(ACTIONS.setLotStatusStart()),
  setLotStatusSuccess: data => dispatch(ACTIONS.setLotStatusSuccess(data)),
  setLotStatusError: error => dispatch(ACTIONS.setLotStatusError(error))
});

export default compose(
  withFirebase,
  connect(
    mapStateToProps,
    mapDispatchToProps
  )
)(RentButton);
