import React, { Component, Fragment } from "react";
import { connect } from "react-redux";
import { compose } from "recompose";
import { withRouter } from "react-router-dom";
import { withFirebase } from "../../components/Firebase";
import { Segment } from "semantic-ui-react";

import * as ROUTES from "../../constants/routes";

import UserDataBlock from "../../components/UserDataBlock";
import UserLotsList from "../../components/UserLotsList";
import LotCard from "../../components/UserLotsList/LotCard";
import LotMap from "../../components/Mapbox/LotMap";
import StructuresList from "../../components/StructuresList";
import PriceHistoryChart from "../../components/PriceHistoryChart";

import LotButtons from "./buttons/LotButtons";

import { setActiveRoute } from "../../actions/page";
import * as LOT_ACTIONS from "../../actions/lot";
import { REFRESH } from "../../constants/lot";

import * as BIT from "../../utils/Bitwise";

import "./Lot.scss";

class LotPage extends Component {
  constructor(params) {
    super(params);

    this.state = {
      lot: null,
      loading: false,
      showStructures: true,
      showPriceHistory: true,
      uid: null,
      showDebug: false
    };
  }

  componentDidMount() {
    const { setActiveRoute } = this.props;
    setActiveRoute(ROUTES.LOT);

    this.fetchData();
  }

  componentDidUpdate() {
    if (BIT.isset(this.props.needRefresh, REFRESH.LOT)) {
      console.log("NEED REFRESH LOT DATA");
      this.fetchData();
    }
  }

  fetchData() {
    const {
      firebase,
      fetchLotDataStart,
      fetchLotDataSuccess,
      fetchLotDataError
    } = this.props;
    const uid = this.currLotUid();

    fetchLotDataStart();
    firebase
      .lotByUid(uid)
      .then(result => {
        try {
          var lots = JSON.parse(result.data.lots);
          fetchLotDataSuccess({ ...lots[uid], uid });
        } catch (error) {
          fetchLotDataError(error);
        }
      })
      .catch(error => {
        fetchLotDataError(error);
      });
  }

  currLotUid() {
    return document.location.pathname.match(/[^/]+$/)[0];
  }

  render() {
    const { lot, loading, authUser, history } = this.props;

    if (!authUser) {
      history.push(ROUTES.SIGN_IN);
      return null;
    }
    if (lot && lot.uid && lot.uid !== this.currLotUid()) {
      document.location.reload();
    }

    return (
      <div className="page table">
        <div className="table-row">
          <div className="table-cell" style={{ width: "65%" }}>
            <LotButtons />
          </div>
          <div className="table-cell" style={{ width: "35%" }}>
            <UserDataBlock />
          </div>
        </div>
        <div className="table-row">
          <div className="table-cell" style={{ width: "65%" }}>
            {lot && (
              <Fragment>
                <LotCard
                  lot={lot}
                  title={true}
                  className="lot-page"
                  link={false}
                />
                <LotMap lot={lot} informer={false} />
                {lot.structures && Object.keys(lot.structures).length > 0 && (
                  <div id="structures" className="section">
                    <h3 className="flex-1">Structures</h3>
                    <StructuresList data={lot.structures} />
                  </div>
                )}
                {/* <div id="priceHistory" className="section">
                  <h3 className="flex-1">Price history</h3>
                  <PriceHistoryChart />
                </div> */}
              </Fragment>
            )}
          </div>
          <div className="table-cell" style={{ width: "35%" }}>
            <UserLotsList />
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state /*, ownProps*/) => {
  const { authUser } = state.session;
  const {
    currentLot,
    error,
    loading,
    lotStatusChanging,
    needRefresh
  } = state.lot;

  return {
    authUser,
    lot: currentLot,
    error,
    loading,
    lotStatusChanging,
    needRefresh
  };
};

const mapDispatchToProps = dispatch => ({
  setActiveRoute: route => dispatch(setActiveRoute(route)),
  fetchLotDataStart: () => dispatch(LOT_ACTIONS.fetchLotDataStart()),
  fetchLotDataSuccess: data => dispatch(LOT_ACTIONS.fetchLotDataSuccess(data)),
  fetchLotDataError: error => dispatch(LOT_ACTIONS.fetchLotDataError(error))
});

export default compose(
  withRouter,
  withFirebase,
  connect(
    mapStateToProps,
    mapDispatchToProps
  )
)(LotPage);
