import React from "react";
import { Link } from "react-router-dom";

export const capitalize = str => str.substr(0, 1).toUpperCase() + str.substr(1);

const REX_ADDRESS = {
  ru: /ул(?:ица|\.)|пр(?:оспект|\.)|пер(?:еулок|\.)|пр(?:оезд|\.|-д)|аллея|бул(?:ьвар|\.)|наб(?:ережная|\.)|ш(?:оссе|\.)|пл(?:ощадь|\.)/i
};

export const addressByLocale = (srcStr = "", locale = "ru") => {
  let address = "",
    region = "";
  switch (locale) {
    case "ru":
      const arr = srcStr.split(/\s*,\s*/);
      const pos = arr.findIndex(v => REX_ADDRESS[locale].test(v));
      const arr2 = arr.splice(0, pos);
      address = arr.join(", ");
      region = arr2.join(", ");
      break;
    default:
      address = srcStr;
  }
  return { address, region };
};

export const addressSpan = ({
  address,
  className = "",
  locale = "ru",
  href = null
}) => {
  if (!address) {
    if (href)
      return (
        <Link to={href}>
          <div className="address_country grey italic">
            Address not specified
          </div>
        </Link>
      );
    else return <div className="grey italic">Address not specified</div>;
  }
  const addr = addressByLocale(address, locale);
  // const arr = address.split(/\s*,\s*/);
  // const pos = arr.findIndex(v => REX_ADDRESS[locale].test(v));
  // const arr2 = arr.splice(0, pos);
  return (
    <span className={`address ${className}`}>
      {(href => {
        if (href)
          return (
            <Link to={href}>
              <span className="address_street">{addr.address}</span>
            </Link>
          );
        else return <span className="address_street">{addr.address}</span>;
      })(href)}

      <br />
      <span className="address_country grey">{addr.region}</span>
    </span>
  );
};

export const money = (amount = 0, currency = "USD") => {
  const sign = amount < 0 ? "-" : "";
  let [int, dec = "0"] = (Math.abs(amount) / 100).toFixed(2).split(".");
  let len = int.length % 3 || 3;
  const acc = [];
  let res = "";
  while (int.length) {
    acc.push(int.slice(0, len));
    int = int.substr(len);
    len = 3;
  }
  switch (currency) {
    case "USD":
      res = `${sign}$${acc.join(",")}.${padStart(dec, 2, "0")}`;
      break;
    case "RUR":
      res = `${sign}${acc.join(" ")},${padStart(dec, 2, "0")} RUR`;
      break;
    default:
      res = `${sign}${acc.join(" ")},${padStart(dec, 2, "0")}`;
  }
  return res;
};

export const padStart = (v = 0, l = 0, c = " ") => {
  let s = "" + v;
  while (s.length < l) {
    s = c + s;
  }
  return s;
};

export const date = (value, forList = false) => {
  const now = destructDate(new Date());
  try {
    const d = destructDate(value instanceof Date ? value : new Date(value));
    if (forList === true) {
      if (isSameDay(d, now))
        return `today, ${padStart(d.hours, 2, "0")}:${padStart(
          d.minutes,
          2,
          "0"
        )}`;
      else if (isSameYear(d, now)) return `${MONTH_NAMES[d.month]} ${d.day}`;
      else
        return `${padStart(d.day, 2, "0")}.${padStart(d.month + 1, 2, "0")}.${
          d.year
        }`;
    } else {
      return `${padStart(d.day, 2, "0")}.${padStart(d.month + 1, 2, "0")}.${
        d.year
      }
      ${padStart(d.hours, 2, "0")}:${padStart(d.minutes, 2, "0")}:${padStart(
        d.seconds,
        2,
        "0"
      )}`;
    }
  } catch (e) {
    console.error(value);
    return "";
  }
};

export const plural = (val, word) => `${val} ${word}${val === 1 ? "" : "s"}`;

export const geoCoords = (point = null) =>
  point && typeof point === "object" && point._latitude && point._longitude
    ? `${point._latitude.toFixed(6)}, ${point._longitude.toFixed(6)}`
    : "";

const destructDate = v => ({
  time: v.getTime(),
  day: v.getDate(),
  month: v.getMonth(),
  year: v.getFullYear(),
  hours: v.getHours(),
  minutes: v.getMinutes(),
  seconds: v.getSeconds()
});

const MONTH_NAMES = [
  "January",
  "February",
  "March",
  "April",
  "May",
  "June",
  "July",
  "August",
  "September",
  "October",
  "November",
  "December"
];

// const DAY = 1000 * 60 * 60 * 24;

const isSameDay = (v1, v2) =>
  v1.day === v2.day && v1.month === v2.month && v1.year === v2.year;
const isSameYear = (v1, v2) => v1.year === v2.year;
// const diffInDays = (v1, v2) => Math.floor((v1.time - v2.time) / DAY);
