// array.sum
if (!Array.prototype.sum) {
  // eslint-disable-next-line
  Array.prototype.sum = function() {
    let acc = 0;
    for (let val of this) {
      if (!isNaN(val)) {
        acc += val;
      }
    }
    return acc;
  };
}

// object.forEach
if (!Object.prototype.forEach) {
  // eslint-disable-next-line
  Object.defineProperty(Object.prototype, "forEach", {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function(callee = () => void 0, scope = null) {
      for (let key in this) {
        if (this.hasOwnProperty(key)) {
          callee.call(scope, this[key], key, this);
        }
      }
    }
  });
}

// object.toArray
if (!Object.prototype.toArray) {
  // eslint-disable-next-line
  Object.defineProperty(Object.prototype, "toArray", {
    enumerable: false,
    configurable: true,
    writable: false,
    value: function(transform = value => value, scope = null) {
      const acc = [];
      for (let key in this) {
        if (this.hasOwnProperty(key)) {
          acc.push(transform.call(scope, this[key], key));
        }
      }
      return acc;
    }
  });
}

// object.watch
if (!Object.prototype.watch) {
  // eslint-disable-next-line
  Object.defineProperty(Object.prototype, "watch", {
    enumerable: false,
    configurable: true,
    writable: false,
    value: function(prop, handler) {
      var oldval = this[prop],
        newval = oldval,
        getter = function() {
          return newval;
        },
        setter = function(val) {
          oldval = newval;
          return (newval = handler.call(this, prop, oldval, val));
        };
      if (delete this[prop]) {
        // can't watch constants
        Object.defineProperty(this, prop, {
          get: getter,
          set: setter,
          enumerable: true,
          configurable: true
        });
      }
    }
  });
}

// object.unwatch
if (!Object.prototype.unwatch) {
  // eslint-disable-next-line
  Object.defineProperty(Object.prototype, "unwatch", {
    enumerable: false,
    configurable: true,
    writable: false,
    value: function(prop) {
      var val = this[prop];
      delete this[prop]; // remove accessors
      this[prop] = val;
    }
  });
}
